// NCPointSequence.cpp: implementation of the CNCPointSequence class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "drill2nc1.h"
#include "NCPointSequence.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define POINT_SEQ_MEM_DELTA 256

static int psid = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNCPointSequence::CNCPointSequence()
:m_PointList(NULL), m_PointCount(0), m_MemCount(0)
{
	m_psid = psid++;
	TRACE("   +++         PS #%02d @ %08p\n", m_psid, this);
}

CNCPointSequence::~CNCPointSequence()
{
	TRACE("   -           PS #%02d @ %08p\n", m_psid, this);
	if (m_PointList)
	{
//		delete m_PointList;
	}
	m_PointList = NULL;
	m_PointCount = 0;
	m_MemCount = 0;
	TRACE("   --          PS #%02d @ %08p\n", m_psid, this);
}

const CNCPoint CNCPointSequence::operator [](int pos)
{
	ASSERT((pos>=0) && (pos<m_PointCount));
	return m_PointList[pos];
}

void CNCPointSequence::IncMem()
{
	CNCPoint *buf = (CNCPoint*)calloc(m_MemCount + POINT_SEQ_MEM_DELTA, sizeof(CNCPoint));//new CNCPoint[m_MemCount + POINT_SEQ_MEM_DELTA];
	ASSERT(buf != NULL);
	m_MemCount += POINT_SEQ_MEM_DELTA;
	for (int pi=0; pi<m_PointCount; pi++) buf[pi] = m_PointList[pi];
	CNCPoint *tmp = m_PointList;
	m_PointList = buf;
	if (tmp) free(tmp);//delete tmp;
}

int CNCPointSequence::Add(CNCPoint p)
{
	if (m_MemCount <= m_PointCount) IncMem();
	m_PointList[m_PointCount] = p;
	return m_PointCount++;
}

void CNCPointSequence::Insert(int pos, CNCPoint p)
{
	ASSERT((pos>=0) && (pos<=m_PointCount));
	if (m_MemCount <= m_PointCount) IncMem();
	for (int pi=m_PointCount; pi>pos; pi++) m_PointList[pi] = m_PointList[pi-1];
	m_PointList[pos] = p;
	m_PointCount++;
}

void CNCPointSequence::Swap(int a, int b)
{
	ASSERT((a>=0) && (a<m_PointCount));
	ASSERT((b>=0) && (b<m_PointCount));
	CNCPoint tmp = m_PointList[a];
	m_PointList[a] = m_PointList[b];
	m_PointList[b] = tmp;
}

void CNCPointSequence::Set(int pos, CNCPoint p)
{
	ASSERT((pos>=0) && (pos<m_PointCount));
	m_PointList[pos] = p;
}

void CNCPointSequence::Delete(int pos)
{
	ASSERT((pos>=0) && (pos<m_PointCount));
	for (int pi=pos+1; pi<m_PointCount; pi++) m_PointList[pi-1] = m_PointList[pi];
}

int CNCPointSequence::GetSize()
{
	return m_PointCount;
}

void CNCPointSequence::operator =(CNCPointSequence src)
{
	int cnt = src.GetSize();
	while (m_MemCount < cnt) IncMem();
	m_PointCount = cnt;
	for (int pi=0; pi<cnt; pi++) m_PointList[pi] = src[pi];
}

void CNCPointSequence::Clear()
{
	m_PointCount = 0;
}

const CNCPoint CNCPointSequence::GetAt(int pos)
{
	ASSERT((pos>=0) && (pos<m_PointCount));
	return m_PointList[pos];
}
