// NCToolList.cpp: implementation of the CNCToolList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "drill2nc1.h"
#include "NCToolList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define TOOL_LIST_MEM_DELTA 256

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNCToolList::CNCToolList()
:m_ToolList(NULL), m_MemCount(0), m_ToolCount(0), m_SelTool(-1), m_SelPoint(-1)
{

}

CNCToolList::~CNCToolList()
{
	if (m_ToolList) free(m_ToolList);//delete m_ToolList;
	m_ToolList = NULL;
	m_MemCount = 0;
	m_ToolCount = 0;
}

void CNCToolList::IncMem()
{
	CNCTool *buf = (CNCTool*)calloc(m_MemCount+TOOL_LIST_MEM_DELTA, sizeof(CNCTool));//new CNCTool[m_MemCount + TOOL_LIST_MEM_DELTA];
	ASSERT(buf != NULL);
	m_MemCount += TOOL_LIST_MEM_DELTA;
	for (int ti=0; ti<m_ToolCount; ti++) buf[ti] = m_ToolList[ti];
	CNCTool *tmp = m_ToolList;
	m_ToolList = buf;
	if (tmp) free(tmp);//delete tmp;
}

int CNCToolList::Add(CNCTool p)
{
	if (m_MemCount <= m_ToolCount) IncMem();
	m_ToolList[m_ToolCount] = p;
	return m_ToolCount++;
}

void CNCToolList::Insert(int pos, CNCTool p)
{
	ASSERT((pos>=0) && (pos<=m_ToolCount));
	if (m_MemCount <= m_ToolCount) IncMem();
	for (int ti=m_ToolCount; ti>pos; ti++) m_ToolList[ti] = m_ToolList[ti-1];
	m_ToolList[pos] = p;
	m_ToolCount++;
}

void CNCToolList::Swap(int a, int b)
{
	ASSERT((a>=0) && (a<m_ToolCount));
	ASSERT((b>=0) && (b<m_ToolCount));
	CNCTool tmp = m_ToolList[a];
	m_ToolList[a] = m_ToolList[b];
	m_ToolList[b] = tmp;
}

void CNCToolList::Set(int pos, CNCTool p)
{
	ASSERT((pos>=0) && (pos<m_ToolCount));
	m_ToolList[pos] = p;
}

void CNCToolList::Delete(int pos)
{
	ASSERT((pos>=0) && (pos<m_ToolCount) && (m_ToolCount>0));
	CNCTool *tool = &m_ToolList[pos];
	for (int ti=pos+1; ti<m_ToolCount; ti++) m_ToolList[ti-1] = m_ToolList[ti];
	tool->Clear();
	m_ToolCount--;
}

int CNCToolList::GetSize()
{
	return m_ToolCount;
}

int CNCToolList::Find(int index)
{
	int pos = -1;
	for (int ti=0; ti<m_ToolCount && pos<0; ti++) if (m_ToolList[ti].m_Index == index) pos = ti;
	return pos;
}

CNCTool* CNCToolList::operator[](int pos)
{
	ASSERT((pos>=0) && (pos<m_ToolCount));
	return &(m_ToolList[pos]);
}

void CNCToolList::DeleteAll()
{
	for (int ti=0; ti<m_ToolCount; ti++) m_ToolList[ti].Clear();
	m_ToolCount = 0;
	m_SelTool = -1;
}

void CNCToolList::CalcRect()
{
	if (m_ToolCount)
	{
		char first_find = true;
		for (int ti=0; ti<m_ToolCount && first_find; ti++)
		{
			if (m_ToolList[ti].GetSize())
			{
				m_TopLeft = m_BottomRight = m_ToolList[ti][0];
				first_find = false;
			}
		}
		for (ti=0; ti<m_ToolCount; ti++)
		{
			for (int pi=m_ToolList[ti].GetSize()-1; pi>=0; pi--)
			{
				if (m_TopLeft.m_X > m_ToolList[ti][pi].m_X) m_TopLeft.m_X = m_ToolList[ti][pi].m_X;
				if (m_TopLeft.m_Y > m_ToolList[ti][pi].m_Y) m_TopLeft.m_Y = m_ToolList[ti][pi].m_Y;
				if (m_BottomRight.m_X < m_ToolList[ti][pi].m_X) m_BottomRight.m_X = m_ToolList[ti][pi].m_X;
				if (m_BottomRight.m_Y < m_ToolList[ti][pi].m_Y) m_BottomRight.m_Y = m_ToolList[ti][pi].m_Y;
			}
		}
	}
	else
	{
		m_TopLeft.m_X = 0;
		m_TopLeft.m_Y = 0;
		m_BottomRight.m_X = 0;
		m_BottomRight.m_Y = 0;
	}
}

CNCTool* CNCToolList::GetAt(int pos)
{
	ASSERT((pos>=0) && (pos<m_ToolCount));
	return &(m_ToolList[pos]);
}
